from .auxfuncs import _Bool, _ROut, _Var

def var2fixfortran(vars: _Var, a: str, fa: str | None = None, f90mode: _Bool | None = None) -> str: ...
def useiso_c_binding(rout: _ROut) -> bool: ...
def createfuncwrapper(rout: _ROut, signature: int = 0) -> str: ...
def createsubrwrapper(rout: _ROut, signature: int = 0) -> str: ...
def assubr(rout: _ROut) -> tuple[dict[str, str], str]: ...
